package net.wanji.common.framework.log;

import com.alibaba.fastjson.JSON;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.annotation.aspect.AspectLog;
import net.wanji.common.dto.OperateLog;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.common.utils.tool.DateUtil;
import net.wanji.common.utils.tool.IpAddressUtil;
import net.wanji.common.utils.tool.StringUtils;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.AfterReturning;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Before;
import org.aspectj.lang.annotation.Pointcut;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.lang.reflect.Method;
import java.util.*;

import static net.wanji.common.framework.Constants.DATE_FORMAT.E_DATE_FORMAT_SECOND;


/**
 * @author duanruiming
 * @date 2023/02/27 9:35
 */
@Aspect
@Slf4j
@Component
public class LogAspectHandler {

    @Value("${spring.application.name}")
    private String serverName;

    /**
     * @AspectLog自定义注解标记的controller
     */
    @Pointcut("@annotation(net.wanji.common.annotation.aspect.AspectLog)")
    public void controllerAspect() {
    }

    /**
     * 执行切点controller之前执行的逻辑：打印请求日志
     *
     * @param joinPoint
     */
    @Before("controllerAspect()")
    public void executeBefore(JoinPoint joinPoint) {
        try {
            HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
            String ip = IpAddressUtil.getIpAddress(request);
            Map<String, String> map = getControllerMethodDescription(joinPoint);
            OperateLog log = new OperateLog();
            log.setDescription(map.get("description").toString());
            log.setIp(ip);
            log.setSystem(serverName);
            log.setOperationType(map.get("operationType").toString());
            log.setUrl(request.getRequestURL().toString());
            log.setArgs(getArgs(request, joinPoint));
            log.setCreateTime(new Date());
            //打印操作日志
            this.consoleLog(log, request);
        } catch (Exception e) {
            log.error("打印日志信息日志切面执行异常，异常信息: {}", e);
        }
    }

    /**
     * MVC返回结果处理逻辑，打印请求信息和结果信息
     * todo 后续可增加日志流水表，插入流水表中
     *
     * @param joinPoint
     * @param jsonViewObject
     */
    @AfterReturning(value = "controllerAspect()", returning = "jsonViewObject")
    public void afterReturning(JoinPoint joinPoint, JsonViewObject jsonViewObject) {
        HttpServletRequest request = ((ServletRequestAttributes)
                RequestContextHolder.getRequestAttributes()).getRequest();
        try {
            //请求的IP
            String ip = IpAddressUtil.getIpAddress(request);
            //url
            String uri = request.getRequestURL().toString();
            //获取信息
            Map<String, String> map = getControllerMethodDescription(joinPoint);
            OperateLog operateLog = new OperateLog();
            operateLog.setDescription(map.get("description").toString());
            operateLog.setIp(ip);
            operateLog.setSystem(serverName);
            operateLog.setOperationType(map.get("operationType").toString());
            operateLog.setUrl(request.getRequestURL().toString());
            operateLog.setArgs(getArgs(request, joinPoint));
            operateLog.setStatus(String.valueOf(jsonViewObject.getCode()));
            operateLog.setResult(JSON.toJSONString(jsonViewObject));
            operateLog.setCreateTime(new Date());
            log.info("\n外部系统调用本地接口，请求IP = {} || 接口 = {} || 地址 = {} || 响应状态 = {}", ip, map.get("description").toString(), uri, jsonViewObject.getCode());
        } catch (Exception e) {
            //记录本地异常日志
            log.error("记录本地接口响应日志异常，异常信息:{}", e);
        }
    }


    /**
     * 日志打印模板
     */
    private String consoleLogTemplate = "\r\n" +
            "系统：%s\r\n" +
            "URL：%s\r\n" +
            "操作类型：%s\r\n" +
            "操作描述：%s\r\n" +
            "操作IP：%s\r\n" +
            "操作时间：%s\r\n" +
            "请求参数：%s\r\n";

    /**
     * 打印输出格式日志
     *
     * @param operateLog 输出日志对象
     * @param request    请求对象
     */
    private void consoleLog(OperateLog operateLog, HttpServletRequest request) {
        try {
            String logText = String.format(consoleLogTemplate,
                    serverName,
                    operateLog.getUrl(),
                    operateLog.getOperationType(),
                    operateLog.getDescription(),
                    operateLog.getIp(),
                    DateUtil.formatDate(operateLog.getCreateTime(), E_DATE_FORMAT_SECOND.getStrFormat()),
                    operateLog.getArgs());
            log.info(logText);
        } catch (Exception e) {
            log.error("打印操作日志异常，异常信息: {}", e);
        }
    }

    /**
     * @param request   请求对象
     * @param joinPoint
     * @return String 请求参数
     * @discription: [获取请求参数]
     */
    public static String getArgs(HttpServletRequest request, JoinPoint joinPoint) {
        Enumeration<String> parameterNames = request.getParameterNames();
        Map<String, Object> requestParams = new HashMap<>();
        while (parameterNames.hasMoreElements()) {
            String argName = parameterNames.nextElement();
            String value = request.getParameter(argName);
            requestParams.put(argName, value);
        }
        if (requestParams.size() <= 0) {
            Object[] args = joinPoint.getArgs();
            for (Object arg : args) {
                if (null == arg) {
                    continue;
                }
                if (arg instanceof HttpServletRequest || arg instanceof HttpServletResponse) {
                    continue;
                }
                String argName = arg.getClass().getName();
                Object value = arg;
                requestParams.put(argName, value);
            }
        }
        return JSON.toJSONString(requestParams);
    }

    /**
     * 获取@AspectLog标注的controller方法注解信息
     *
     * @param joinPoint
     * @return
     * @throws Exception
     */
    private Map<String, String> getControllerMethodDescription(JoinPoint joinPoint) throws Exception {
        String targetClassName = joinPoint.getTarget().getClass().getName();
        String methodName = joinPoint.getSignature().getName();
        Object[] args = joinPoint.getArgs();
        Class<?> targetClass = Class.forName(targetClassName);
        Method[] methods = targetClass.getMethods();
        Map<String, String> map = new HashMap<>();
        for (Method method : methods) {
            if (StringUtils.equalsIgnoreCase(methodName, method.getName())) {
                Class<?>[] parameterTypes = method.getParameterTypes();
                if (Objects.equals(parameterTypes.length, args.length)) {
                    String description = method.getAnnotation(AspectLog.class).description();
                    String operationType = method.getAnnotation(AspectLog.class).operationType().toString();
                    map.put("description", description);
                    map.put("operationType", operationType);
                    break;
                }
            }
        }
        return map;
    }

}
